/* arquivo.cpp - biblioteca para manuseamento do arquivo da agenda */

#include <stdio.h>
#include <iostream.h>
#include <process.h>
#include <stdlib.h>
#include <string.h>

#include "arquivo.h"

/*--------------------------------------------------------------------------*\
 | disco::disco - Contrutor da classe disco, inicializa os campos da classe.|
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

disco::disco (void)

{

  fp_arq_ind  = NULL;
  fp_arq_base = NULL;

  cpz_identif_arq_ind  = NULL;
  cpz_identif_arq_base = NULL;

}

/*--------------------------------------------------------------------------*\
 | disco::disco - Contrutor da classe disco, abre os arquivos de leitura    |
 |                (arquivo de indice e da base de dados.                    |
 |                                                                          |
 | Recebe - Nome do arquivo de indice e o da base.                          |
 |                                                                          |
 | Retorna - Nada.                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

disco::disco (const char *cpz_nome_arq_ind, const char *cpz_nome_arq_base)

{

  if ((fp_arq_ind = fopen (cpz_nome_arq_ind, "r+b")) == NULL) {
    cerr << "Nao consequi abrir o arquivo " << cpz_nome_arq_ind << "\n";
    exit (EXIT_FAILURE);
  }

  cpz_identif_arq_ind = new char [strlen (cpz_nome_arq_ind) + 1];
  strcpy (cpz_identif_arq_ind, cpz_nome_arq_ind);

  if ((fp_arq_base = fopen (cpz_nome_arq_base, "r+b")) == NULL) {
    cerr << "Nao consegui abrir o arquivo " << cpz_nome_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

  cpz_identif_arq_base = new char [strlen (cpz_nome_arq_base) + 1];
  strcpy (cpz_identif_arq_base, cpz_nome_arq_base);

}

/*--------------------------------------------------------------------------*\
 | disco::crie_arquivos - Constroi entradas no diretorio para o arquivo de  |
 |                        indice e para o arquivo de base.                  |
 |                                                                          |
 | Recebe - O nome do arquivo de indice e o nome do arquivo de base.        |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void disco::crie_arquivos (const char *cpz_nome_arq_ind,
			   const char *cpz_nome_arq_base)
{

  if ((fp_arq_ind = fopen (cpz_nome_arq_ind, "wb")) == NULL) {
    cerr << "Nao consequi abrir o arquivo " << cpz_nome_arq_ind << "\n";
    exit (EXIT_FAILURE);
  }

  fclose (fp_arq_ind);

  fp_arq_ind = NULL;

  if ((fp_arq_base = fopen (cpz_nome_arq_base, "wb")) == NULL) {
    cerr << "Nao consegui abrir o arquivo " << cpz_nome_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

  fclose (fp_arq_base);

  fp_arq_base = NULL;

}

/*--------------------------------------------------------------------------*\
 | disco::leia_arq_ind - Le o proximo indicado.                             |
 |                                                                          |
 | Recebe - Um buffer para receber as informacoes e o tamanho do registro   |
 |          de indice e a posicao desejada.                                 |
 |                                                                          |
 | Retorna - O buffer preenchido com as informacoes.                        |
 |                                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

void disco::leia_arq_ind (char *cpz_buffer, const int i_tam,
                         const int i_posicao)

{

  fseek (fp_arq_ind, (long) (i_posicao - 1) * (long) i_tam, SEEK_SET);

  if (fread (cpz_buffer, i_tam, 1, fp_arq_ind) == 0) {
    cerr << "Erro no arquivo " << cpz_identif_arq_ind << "\n";
    exit (EXIT_FAILURE);
  }

}

/*--------------------------------------------------------------------------*\
 | disco::leia_arq_ind - Le o proximo registro valido dentro do arquivo de  |
 |                       indice. Se a leitura for bem sucedida entao a      |
 |                       funcao retorna a posicao do registro lido. Retorna |
 |                       erro no caso de uma leitura mal sucedida.          |
 |                                                                          |
 | Recebe - Um buffer para receber as informacoes e o tamanho do registro   |
 |          de indice.                                                      |
 |                                                                          |
 | Retorna - O buffer preenchido com as informacoes e um inteiro represen-  |
 |           ando a posicao do registro lido ou erro se houver problemas.   |
 |                                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

int disco::leia_arq_ind (char *cpz_buffer, const int i_tam)

{

  long l_posicao;   // Posicao do ponteiro do arquivo

  int i_lidos;      // Numero de bytes lidos pela funcao de leitura


  do {

    l_posicao = ftell (fp_arq_ind);

    i_lidos = fread (cpz_buffer, i_tam, 1, fp_arq_ind);

  } while (!strncmp (cpz_buffer, DELETADO, strlen (DELETADO)) &&
	   i_lidos != 0);

  l_posicao /= i_tam;

  if (i_lidos != 0)
    return ((int) l_posicao + 1);
  else
    return ERRO;

}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_ind - Grava o registro de indice na primeira posicao    |
 |         		  valida dentro do arquivo e retorna a posicao      |
 |                        em que ele foi colocado.                          |
 |                                                                          |
 | Recebe - Um ponteiro para o buffer que contem o registro, e um inteiro   |
 |          contendo o tamanho do registro a ser gravado no arquivo.        |
 |                                                                          |
 | Retorna - A posicao em que ele foi colocado.            (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

int disco::grave_arq_ind (int num_regs, char *cpz_buffer, const int i_tam)

{

  char *cpz_buf_verif = new char [i_tam + 1]; // buffer temporario

  int i_lidos;

  long l_posic;

  long l_tam_arq;

  fseek (fp_arq_ind, 0L, SEEK_END);
  l_posic = l_tam_arq = ftell (fp_arq_ind);

  if ((l_tam_arq / i_tam) == num_regs) {
    l_posic /= (long) i_tam;

    fwrite (cpz_buffer, i_tam, 1, fp_arq_ind);

    delete cpz_buf_verif;
    return ((int) l_posic + 1);
  }


  fseek (fp_arq_ind, 0L, SEEK_SET);

  do {

    l_posic = ftell (fp_arq_ind);

    i_lidos = fread (cpz_buf_verif, i_tam, 1, fp_arq_ind);

  } while (strncmp (cpz_buf_verif, DELETADO, strlen (DELETADO)) &&
	 i_lidos != 0);

  delete cpz_buf_verif;

  if (i_lidos != 0)                               // se ha registro deletado
    fseek (fp_arq_ind, (long) -i_tam, SEEK_CUR);  //   grave nele.

  l_posic /= (long) i_tam;

  fwrite (cpz_buffer, i_tam, 1, fp_arq_ind);

  return ((int) l_posic + 1);

}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_ind - Grava o registro de indice em uma posicao         |
 |         		  determinada.                                      |
 |                                                                          |
 | Recebe - Um ponteiro para o buffer que contem o registro, e um inteiro   |
 |          contendo o tamanho do registro e a posicao no arquivo.          |
 |                                                                          |
 | Retorna - Nada.                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

void disco::grave_arq_ind (char *cpz_buffer, const int i_posicao,
			   const int i_tam)
{

  fseek (fp_arq_ind, (long) (i_posicao - 1) * (long) i_tam, SEEK_SET);

  fwrite (cpz_buffer, i_tam, 1, fp_arq_ind);

}

/*--------------------------------------------------------------------------*\
 | disco::delete_reg_arq_ind - marca o registro ind como deletado.          |
 |                                                                          |
 | Recebe - O tamanho de cada registro e a posicao do registro a ser        |
 |          deletado.                                                       |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void disco::delete_reg_arq_ind (const int i_tam, const int i_posic)

{

  char *cpz_buffer = new char [i_tam + 1];


  strcpy (cpz_buffer, DELETADO);

  for (int i = strlen (DELETADO) + 1; i < i_tam; i++)
    cpz_buffer [i] = ' ';
  cpz_buffer [i_tam + 1] = '\0';

  fseek (fp_arq_ind, (long) i_tam * (long) (i_posic - 1), SEEK_SET);

  if (fwrite (cpz_buffer, i_tam, 1, fp_arq_ind) == 0) {
    delete cpz_buffer;
    cerr << "Erro no arquivo " << cpz_identif_arq_ind << "\n";
    exit (EXIT_FAILURE);
  }

  delete cpz_buffer;

}

/*--------------------------------------------------------------------------*\
 | disco::leia_arquivo_base - executa a leitura de um registro no arquivo   |
 |                            da base de dados.                             |
 |                                                                          |
 | Recebe - Um buffer contendo o registro, o seu tamanho e a posicao que em |
 |          que se encontra.                                                |
 |                                                                          |
 | Retorna - O registro lido.                                               |
\*--------------------------------------------------------------------------*/

void disco::leia_arq_base (char *cpz_buffer, const int i_tam,
			   const int i_posic)
{

  fseek (fp_arq_base, (long) i_tam * (long) (i_posic - 1), SEEK_SET);

  if (fread (cpz_buffer, i_tam, 1, fp_arq_base) == 0) {
    cerr << "Erro no arquivo " << cpz_identif_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_base - grava o registro de base na primeira posicao     |
 |                         valida do arquivo, retornando entao a posicao    |
 |                         em que foi colocado o registro.                  |
 |                                                                          |
 | Recebe - Um buffer com o registro e um inteiro com o tamanho do mesmo.   |
 |                                                                          |
 | Retorna - Um inteiro que indica a posicao no arquivo em que foi colocado |
 |           o registro.                                   (Sobrecarregada) |
\*--------------------------------------------------------------------------*/


int disco::grave_arq_base (int num_regs, char *cpz_buffer, const int i_tam)

{

  char *cpz_buf_verif = new char [i_tam + 1]; // buffer temporario

  int i_lidos;

  long l_posic;

  long l_tam_arq;

  fseek (fp_arq_base, 0L, SEEK_END);
  l_posic = l_tam_arq = ftell (fp_arq_base);

  if ((l_tam_arq / i_tam) == num_regs) {
    l_posic /= (long) i_tam;

    fwrite (cpz_buffer, i_tam, 1, fp_arq_base);

    delete cpz_buf_verif;

    return ((int) l_posic + 1);
  }


  fseek (fp_arq_base, 0L, SEEK_SET);

  do {

    l_posic = ftell (fp_arq_base);

    i_lidos = fread (cpz_buf_verif, i_tam, 1, fp_arq_base);

  } while (strncmp (cpz_buf_verif, DELETADO, strlen (DELETADO)) &&
	 i_lidos != 0);

  delete cpz_buf_verif;

  if (i_lidos != 0)                               // se ha registro deletado
    fseek (fp_arq_base, (long) -i_tam, SEEK_CUR); //   grave nele.

  fwrite (cpz_buffer, i_tam, 1, fp_arq_base);

  l_posic /= i_tam;

  return ((int) l_posic + 1);

}

/*--------------------------------------------------------------------------*\
 | disco::grave_arq_base - grava o registro de base na posicao indicada.    |
 |                                                                          |
 | Recebe - Um buffer com o registro e dois inteiros, um com a posicao em   |
 |          que se deve gravar o registro e o outro com o tamanho do mesmo. |
 |                                                                          |
 | Retorna - Nada.                                                          |
 |                                                         (Sobrecarregada) |
\*--------------------------------------------------------------------------*/

void disco::grave_arq_base (char *cpz_buffer, const int i_posicao,
			    const int i_tam)
{

  fseek (fp_arq_base, (long) (i_posicao - 1) * (long) i_tam, SEEK_SET);

  fwrite (cpz_buffer, i_tam, 1, fp_arq_base);

}

/*--------------------------------------------------------------------------*\
 | disco::delete_reg_arq_base - marca o registro base como deletado.        |
 |                                                                          |
 | Recebe - O tamanho de cada registro e a posicao do registro a ser        |
 |          deletado.                                                       |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void disco::delete_reg_arq_base (const int i_tam, const int i_posic)

{

  char *cpz_buffer = new char [i_tam + 1];


  strcpy (cpz_buffer, DELETADO);

  for (int i = strlen (DELETADO) + 1; i < i_tam; i++)
    cpz_buffer [i] = ' ';
  cpz_buffer [i_tam + 1] = '\0';

  fseek (fp_arq_base, (long) i_tam * (long) (i_posic - 1), SEEK_SET);

  if (fwrite (cpz_buffer, i_tam, 1, fp_arq_base) == 0) {
    delete cpz_buffer;
    cerr << "Erro no arquivo " << cpz_identif_arq_base << "\n";
    exit (EXIT_FAILURE);
  }

  delete cpz_buffer;

}


/*--------------------------------------------------------------------------*\
 | disco::~disco - fecha os arquivos e elimina o nome dos arquivos.         |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

disco::~disco (void)

{

  if (fp_arq_ind != NULL)
    fclose (fp_arq_ind);

  if (fp_arq_base != NULL)
    fclose (fp_arq_base);

  if (cpz_identif_arq_ind != NULL)
    delete cpz_identif_arq_ind;

  if (cpz_identif_arq_base != NULL)
    delete cpz_identif_arq_base;

};
