/* memoria.cpp - contrele de busca das entidades em memoria */

#include <iostream.h>
#include <stdlib.h>
#include <process.h>
#include <string.h>
#include <alloc.h>

#include "arquivo.h"
#include "registro.h"
#include "memoria.h"

/*--------------------------------------------------------------------------*\
 | memoria::bate - Compara a entidade com a sequencia desejada.             |
 |                                                                          |
 | Recebe - A entidade e a informacao para a busca.                         |
 |                                                                          |
 | Retorna - Ok se bateu our ERRO caso contrario.                           |
\*--------------------------------------------------------------------------*/

int memoria::bate (char far *cpz_entidade, const char *cpz_inf_busca)

{
  int i_tam = strlen (cpz_inf_busca),
      i_tam_entidade = _fstrlen (cpz_entidade),

      i_bateu,
      i, j;

  for (i = 0; i < i_tam_entidade - i_tam + 1; i++) {

    i_bateu = OK;

    for (j = 0; j < i_tam; j++)
      if (cpz_entidade [i + j] != cpz_inf_busca [j]) {
	i_bateu = ERRO;
	break;
      }

    if (i_bateu == OK)
      return OK;
  }

  return (ERRO);
}

/*--------------------------------------------------------------------------*\
 | memoria::memoria - Construtor da classe memoria. Inicializacao de        |
 |                    variaveis.                                            |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retrorna - Nada.                                                         |
\*--------------------------------------------------------------------------*/

memoria::memoria (void)

{
  ptr_chaves = NULL;


  i_num_chaves = 0;
}

/*--------------------------------------------------------------------------*\
 | memoria::num_registros - Informa o numero de registros ativos.           |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retrorna - O numero de registros ativos.                                 |
\*--------------------------------------------------------------------------*/

int memoria::num_registros (void)

{
  return (i_num_chaves);
}


/*--------------------------------------------------------------------------*\
 | memoria::insere_chave - Coloca a entidade na pilha.                      |
 |                                                                          |
 | Recebe - A entidade e o indice do arquivo de base.                       |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void memoria::insere_chave (const char *cpz_entidade, const int i_indice_ind,
			    const int i_indice_base)

{

  ptr_nodo novo;


  if ((novo = (ptr_nodo) farmalloc (sizeof (struct nodo))) == NULL) {
    cerr << "Erro: Falta memoria para trabalhar.\n";
    exit (EXIT_FAILURE);
  };

  if ((novo->cpz_entidade = (char far *)farmalloc (strlen (cpz_entidade) + 1))
       == NULL) {
    cerr << "Erro: Falta memoria para trabalhar.\n";
    exit (EXIT_FAILURE);
  };

  _fstrcpy (novo->cpz_entidade, (char far *)cpz_entidade);
  novo->i_ind_arq_ind = i_indice_ind;
  novo->i_ind_arq_base = i_indice_base;

  novo->proximo = ptr_chaves;
  ptr_chaves = novo;

  i_num_chaves++;
}

/*--------------------------------------------------------------------------*\
 | memoria::insere_chave - Coloca a entidade na pilha.                      |
 |                                                                          |
 | Recebe - A entidade e o indice do arquivo de base.                       |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void memoria::retira_chave (const int i_indice_ind, const int i_indice_base)

{

  ptr_nodo ret_nodo_ant, ret_nodo;

  if (ptr_chaves == NULL) {
    cerr << "Erro. A pilha de chaves esta vazia.\n";
    exit (EXIT_FAILURE);
  }

  if (ptr_chaves->i_ind_arq_ind == i_indice_ind &&
      ptr_chaves->i_ind_arq_base == i_indice_base) {
    ret_nodo = ptr_chaves;
    ptr_chaves = ptr_chaves->proximo;
    farfree (ret_nodo->cpz_entidade);
    farfree (ret_nodo);
    i_num_chaves--;
    return;
  }

  ret_nodo = ptr_chaves->proximo;
  ret_nodo_ant = ptr_chaves;

  while (ret_nodo != NULL) {

    if (ret_nodo->i_ind_arq_ind == i_indice_ind &&
	ret_nodo->i_ind_arq_base == i_indice_base) {
      ret_nodo_ant->proximo = ret_nodo->proximo;
      farfree (ret_nodo->cpz_entidade);
      farfree (ret_nodo);
      i_num_chaves--;
      return;
    }

    ret_nodo = ret_nodo->proximo;
    ret_nodo_ant = ret_nodo_ant->proximo;
  }

  cerr << "Erro. Registro nao encontrado na pilha\n";
  exit (EXIT_FAILURE);

}

/*--------------------------------------------------------------------------*\
 | memoria::procura_chave - Executa uma busca na pilha para achar os        |
 |                          registros procurados.                           |
 |                                                                          |
 | Recebe - A sequencia de caracteres com a informacao para a busca.        |
 |                                                                          |
 | Retorna - Ok se acho, ERRO se terminou a busca. Se acho retorna a        |
 |           Entidade e o indice do arquivo de base.                        |
\*--------------------------------------------------------------------------*/

int memoria::procura_chave (char *cpz_entidade, int &i_indice_ind,
			    int &i_indice_base, const char *cpz_inf_busca)
{

  while (busca != NULL) {

    if (bate (busca->cpz_entidade, cpz_inf_busca)) {
      _fstrcpy ((char far *)cpz_entidade, busca->cpz_entidade);
      i_indice_ind = busca->i_ind_arq_ind;
      i_indice_base = busca->i_ind_arq_base;
      busca = busca->proximo;
      return OK;
    }

    busca = busca->proximo;
  }

  return ERRO;
}

/*--------------------------------------------------------------------------*\
 | memoria::ordene - Coloca a lista de elementos em ordem.                  |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void memoria::ordene (void)
{
  ptr_nodo anter, atual, post = NULL;
  int mudou = FALSO;

  if (ptr_chaves == NULL)
    return;
  if (ptr_chaves->proximo == NULL)
    return;
  if (ptr_chaves->proximo->proximo == NULL) {
    atual = ptr_chaves;
    post = ptr_chaves->proximo;
    if (_fstrcmp (atual->cpz_entidade, post->cpz_entidade) > 0) {
      ptr_chaves = post;
      post->proximo = atual;
      atual->proximo = NULL;
      return;
    }
  }
  do {
    mudou = FALSO;
    atual = ptr_chaves;
    post = ptr_chaves->proximo;
    if (_fstrcmp (atual->cpz_entidade, post->cpz_entidade) > 0) {
      ptr_chaves = post;
      atual->proximo = post->proximo;
      post->proximo = atual;
    }
    anter = ptr_chaves;
    atual = ptr_chaves->proximo;
    post = ptr_chaves->proximo->proximo;
    while (post != NULL) {
      if (_fstrcmp (atual->cpz_entidade, post->cpz_entidade) > 0) {
	atual->proximo = post->proximo;
	post->proximo = atual;
	anter->proximo = post;
	post = atual;
	atual = anter->proximo;
	mudou = VERDADEIRO;
      }
      anter = anter->proximo;
      atual = atual->proximo;
      post = post->proximo;
    }
  } while (mudou);
}

void memoria::liste_memoria (void)
{

  disco entrada (NOME_ARQ_INDICE, NOME_ARQ_BASE);
  ptr_nodo atual = ptr_chaves;
  registro listar_reg;
  int linha = 0, cont;

//  fprintf (stdprn, "%c%c%c%c", 0x1b, 0x5b, 0x37, 0x77); /* Comprime */
  for (char letra = 'A'; letra <= 'Z'; letra++) {
    atual = ptr_chaves;
    if (linha > 0 && linha < 22)
      for (; linha < 22; linha++)
	fprintf (stdprn, "\n");
    linha = 0;
    while (atual != NULL) {
      if (linha == 0 || linha == 20) {
	if (linha != 0)
	  for (; linha < 22; linha++)
	    fprintf (stdprn, "\n");
	for (int tab = 0; tab < 75; tab++)
	  fputc (' ', stdprn);
	fprintf (stdprn, "[%c]\n", letra);
	linha = 1;
      }

      if (letra == atual->cpz_entidade [0]) {
	char *telefone,
	     *buffer_base = new char [listar_reg.tamanho_reg_base () + 1];

	entrada.leia_arq_base (buffer_base, listar_reg.tamanho_reg_base (),
			   atual->i_ind_arq_base);
	listar_reg.coloque_buffer_base (buffer_base);

	telefone = listar_reg.traga_telefone ();

	char prn_entidade [41];
	_fstrncpy ( (char far *)prn_entidade, atual->cpz_entidade, 40);
	int p = 40;
	for (prn_entidade [p] = '\0', p--; p > strlen (prn_entidade) - 1; p--)
	  prn_entidade [p] = '.';

	fprintf (stdprn, "   %-40s.....%-30Fs\n",
			prn_entidade,
			(char far *) telefone);
	linha++;
	delete telefone;
	delete buffer_base;
      }
      atual = atual->proximo;
    }
  }
  atual = ptr_chaves;
  if (linha > 0 && linha < 21)
    for (; linha < 22; linha++)
      fprintf (stdprn, "\n");
  linha = 0;
  fprintf (stdprn, "Registros Invalidos...\n");
  while (atual != NULL) {
    if (atual->cpz_entidade [0] < 'A' || atual->cpz_entidade [0] > 'Z' ) {
      char *telefone,
	   *buffer_base = new char [listar_reg.tamanho_reg_base () + 1];

      entrada.leia_arq_base (buffer_base, listar_reg.tamanho_reg_base (),
			 atual->i_ind_arq_base);
      listar_reg.coloque_buffer_base (buffer_base);

      telefone = listar_reg.traga_telefone ();

      fprintf (stdprn, "     %-30Fs %-30Fs\n", atual->cpz_entidade,
					       (char far *) telefone);
      linha++;
      delete telefone;
      delete buffer_base;
    }
    atual = atual->proximo;
  }
//  fprintf (stdprn, "%c%c%c%c", 0x1b, 0x5b, 0x34, 0x77); /* Descomprime */

}
/*--------------------------------------------------------------------------*\
 | memoria::~memoria - Destroi a pilha com as chaves.                       |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

memoria::~memoria (void)

{
  while (ptr_chaves != NULL) {

    ptr_nodo atual = ptr_chaves;

    ptr_chaves = ptr_chaves->proximo;

    farfree (atual->cpz_entidade);
    farfree (atual);

  }
}