/* registro.cpp - Manuseio de buffer e campos do registro */

#include <iostream.h>
#include <process.h>
#include <stdlib.h>
#include <string.h>

#include "registro.h"

/*--------------------------------------------------------------------------*\
 | registro::coloque_buffer_ind - Separa o buffer recebido nos campos do    |
 |                                registro de indice.                       |
 |                                                                          |
 | Recebe - Um ponteiro para o buffer.                                      |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void registro::coloque_buffer_ind (char *cpz_buffer)

{
  /* copia o nome da entidade */

  for (int i_entidade = 0; i_entidade < TAM_ENTIDADE; i_entidade++)
    cpz_nome_entidade [i_entidade] = cpz_buffer [i_entidade];

  /* copia o indice */

  char cpz_posicao_base [TAM_IND_BASE + 1];

  for (int i = TAM_ENTIDADE, j = 0; i < TAM_ENTIDADE + TAM_IND_BASE; i++, j++)
    cpz_posicao_base [j] = cpz_buffer [i];
  cpz_posicao_base [TAM_IND_BASE] = NULO;

  if ((i_posicao_base = atoi (cpz_posicao_base)) == 0) {
    cerr << "Erro no registro de indice \n";
    cerr << "Indice = " << cpz_posicao_base;
    exit (EXIT_FAILURE);
  }
}

/*--------------------------------------------------------------------------*\
 | registro::coloque_buffer_base - Separa o buffer recebido nos campos do   |
 |                                 registro de base.                        |
 |                                                                          |
 | Recebe - Um ponteiro para o buffer.                                      |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void registro::coloque_buffer_base (char *cpz_buffer)

{

  /* Copia o telefone */

  for (int i = 0; i < TAM_TELEFONE; i++)
    cpz_telefone [i] = cpz_buffer [i];

  /* Copia as linhas */

  for (int i_linha = 0; i_linha < 4; i_linha++)
    for (int i_posic = 0; i_posic < TAM_LINHA; i_posic++)
      cpz_linha_base [i_linha] [i_posic] =
      cpz_buffer [TAM_TELEFONE + i_linha * TAM_LINHA + i_posic];

}

/*--------------------------------------------------------------------------*\
 | registro::traga_buffer_ind - Reune os campos do registro em um buffer.   |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Um buffer preenchido.                                          |
\*--------------------------------------------------------------------------*/

char *registro::traga_buffer_ind (void)

{

  char *cpz_buffer = new char [tamanho_reg_ind () + 1];

  /* copia o nome da entidade */

  for (int i_entidade = 0; i_entidade < TAM_ENTIDADE; i_entidade++)
    cpz_buffer [i_entidade] = cpz_nome_entidade [i_entidade];

  /* copia o indice */

  char cpz_posicao_base [TAM_IND_BASE + 1];

  itoa (i_posicao_base, cpz_posicao_base, 10);

  for (int i_res = strlen (cpz_posicao_base); i_res < TAM_IND_BASE; i_res++)
    cpz_posicao_base [i_res] = ' ';

  for (int i = TAM_ENTIDADE, j = 0; i < TAM_ENTIDADE + TAM_IND_BASE; i++, j++)
    cpz_buffer [i] = cpz_posicao_base [j];
  cpz_buffer [TAM_ENTIDADE + TAM_IND_BASE] = NULO;

  return cpz_buffer;
}

/*--------------------------------------------------------------------------*\
 | registro::traga_buffer_base - Reune os campos do registro em um buffer.  |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - Um buffer preenchido.                                          |
\*--------------------------------------------------------------------------*/

char *registro::traga_buffer_base (void)

{

  char *cpz_buffer = new char [tamanho_reg_base () + 1];

  /* copia o telefone */

  for (int i = 0; i < TAM_TELEFONE; i++)
    cpz_buffer [i] = cpz_telefone [i];

  /* copia as linhas */

  for (int i_linha = 0; i_linha < 4; i_linha++)
    for (int i_posic = 0; i_posic < TAM_LINHA; i_posic++)
      cpz_buffer [TAM_TELEFONE + i_linha * TAM_LINHA + i_posic] =
      cpz_linha_base [i_linha] [i_posic];
  cpz_buffer [TAM_TELEFONE + 4 * TAM_LINHA] = NULO;

  return cpz_buffer;
}


/*--------------------------------------------------------------------------*\
 | registro::coloque_nome_entidade - preenche o campo do nome da entidade.  |
 |                                                                          |
 | Recebe - O nome da entidade.                                             |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void registro::coloque_nome_entidade (const char *cpz_nome_entidade_param)

{

  strcpy (cpz_nome_entidade, cpz_nome_entidade_param);

  for (int i = strlen (cpz_nome_entidade_param); i < TAM_ENTIDADE; i++)
    cpz_nome_entidade [i] = ESPACO;

}

/*--------------------------------------------------------------------------*\
 | registro::coloque_nome_telefone - preenche o campo do telefone.          |
 |                                                                          |
 | Recebe - O telefone.                                                     |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void registro::coloque_telefone (const char *cpz_telefone_param)

{

  strcpy (cpz_telefone, cpz_telefone_param);

  for (int i = strlen (cpz_telefone_param); i < TAM_TELEFONE; i++)
    cpz_telefone [i] = ESPACO;

}


/*--------------------------------------------------------------------------*\
 | registro::coloque_linha - preenche o campo da linha especificada.        |
 |                                                                          |
 | Recebe - Uma linha preenchida.                                           |
 |                                                                          |
 | Retorna - Nada.                                                          |
\*--------------------------------------------------------------------------*/

void registro::coloque_linha (const int i_linha,
			      const char *cpz_linha_base_param)

{

  strcpy (cpz_linha_base [i_linha], cpz_linha_base_param);

  for (int i = strlen (cpz_linha_base_param); i < TAM_LINHA; i++)
    cpz_linha_base [i_linha] [i] = ESPACO;

}

/*--------------------------------------------------------------------------*\
 | registro::traga_nome_entidade - informa o conteudo do campo entidade.    |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - O conteudo do campo entidade.                                  |
\*--------------------------------------------------------------------------*/

char *registro::traga_nome_entidade (void)

{
  char *cpz_nome_entidade_ret = new char [TAM_ENTIDADE + 1];

  strncpy (cpz_nome_entidade_ret, cpz_nome_entidade, TAM_ENTIDADE);

  for (int i = TAM_ENTIDADE - 1; i > 0; i--)
    if (cpz_nome_entidade_ret [i] != ' ')
      break;
  cpz_nome_entidade_ret [++i] = NULO;

  return cpz_nome_entidade_ret;
}


/*--------------------------------------------------------------------------*\
 | registro::traga_telefone - informa o conteudo do campo telefone.         |
 |                                                                          |
 | Recebe - Nada.                                                           |
 |                                                                          |
 | Retorna - O conteudo do campo telefone.                                  |
\*--------------------------------------------------------------------------*/

char *registro::traga_telefone (void)

{
  char *cpz_telefone_ret = new char [TAM_TELEFONE + 1];

  strncpy (cpz_telefone_ret, cpz_telefone, TAM_TELEFONE);
  cpz_telefone_ret [TAM_TELEFONE] = NULO;

  for (int i = TAM_TELEFONE - 1; i > 0; i--)
    if (cpz_telefone_ret [i] != ' ')
      break;
  cpz_telefone_ret [++i] = NULO;


  return cpz_telefone_ret;
}



/*--------------------------------------------------------------------------*\
 | registro::traga_linha - informa o conteudo de uma linha.                 |
 |                                                                          |
 | Recebe - O numero da linha.                                              |
 |                                                                          |
 | Retorna - O conteudo da linha especificada.                              |
\*--------------------------------------------------------------------------*/

char *registro::traga_linha (const int i_linha)

{
  char *cpz_linha_base_ret = new char [TAM_LINHA + 1];

  strncpy (cpz_linha_base_ret, cpz_linha_base [i_linha], TAM_LINHA);
  cpz_linha_base_ret [TAM_LINHA] = NULO;

  for (int i = TAM_LINHA - 1; i > 0; i--)
    if (cpz_linha_base_ret [i] != ' ')
      break;
  cpz_linha_base_ret [++i] = NULO;

  return cpz_linha_base_ret;
}

